@@ -15,17 +15,13 @@
 from collections import Counter
 from collections import OrderedDict
 from collections import abc
-from collections import defaultdict
 from copy import copy
 from glob import glob
 from itertools import chain
 from itertools import count
 from itertools import zip_longest
 from logging import getLogger
-from typing import Dict
-from typing import Iterable
-from typing import List
-from typing import Tuple
+import typing as t
 import hashlib
 import importlib.util
 import os
@@ -73,7 +69,7 @@ def get_migration_hash(migration_id):
 
 
 # eg: "-- depends: 1 2"
-DirectivesType = Dict[str, str]
+DirectivesType = t.Dict[str, str]
 
 LeadingCommentType = str
 
@@ -82,7 +78,7 @@ SqlType = str
 
 def parse_metadata_from_sql_comments(
     s: str,
-) -> Tuple[DirectivesType, LeadingCommentType, SqlType]:
+) -> t.Tuple[DirectivesType, LeadingCommentType, SqlType]:
     directive_names = ["transactional", "depends"]
     comment_or_empty = re.compile(r"^(\s*|\s*--.*)$").match
     directive_pattern = re.compile(
@@ -91,7 +87,7 @@ def parse_metadata_from_sql_comments(
 
     lineending = re.search(r"\n|\r\n|\r", s + "\n").group(0)  # type: ignore
     lines = iter(s.split(lineending))
-    directives = {}  # type: DirectivesType
+    directives: DirectivesType = {}
     leading_comments = []
     sql = []
     for line in lines:
@@ -118,17 +114,19 @@ def parse_metadata_from_sql_comments(
 
 def read_sql_migration(
     path: str,
-) -> Tuple[DirectivesType, LeadingCommentType, List[str]]:
-    directives = {}  # type: DirectivesType
+) -> t.Tuple[DirectivesType, LeadingCommentType, t.List[str]]:
+    directives: DirectivesType = {}
     leading_comment = ""
     statements = []
     if os.path.exists(path):
         with open(path, "r", encoding="UTF-8") as f:
             statements = sqlparse.split(f.read())
             if statements:
-                (directives, leading_comment, sql,) = parse_metadata_from_sql_comments(
-                    statements[0]
-                )
+                (
+                    directives,
+                    leading_comment,
+                    sql,
+                ) = parse_metadata_from_sql_comments(statements[0])
                 statements[0] = sql
     statements = [s for s in statements if s.strip()]
     return directives, leading_comment, statements
@@ -136,7 +134,7 @@ def read_sql_migration(
 
 class Migration(object):
 
-    __all_migrations = {}  # type: Dict[str, "Migration"]
+    __all_migrations: t.Dict[str, "Migration"] = {}
 
     def __init__(self, id, path, source_dir):
         self.id = id
@@ -177,6 +175,7 @@ class Migration(object):
             self.module = types.ModuleType(self.path)
         else:
             spec = importlib.util.spec_from_file_location(self.path, self.path)
+            assert spec is not None
             self.module = importlib.util.module_from_spec(spec)
 
         self.module.step = collector.add_step  # type: ignore
@@ -211,7 +210,9 @@ class Migration(object):
 
         else:
             try:
-                if spec.loader is None:
+                if spec and spec.loader:
+                    spec.loader.exec_module(self.module)
+                else:
                     logger.exception(
                         "Could not import migration from %r: "
                         "ModuleSpec has no loader attached",
@@ -219,8 +220,6 @@ class Migration(object):
                     )
                     raise exceptions.BadMigration(self.path)
 
-                spec.loader.exec_module(self.module)  # type: ignore
-
             except Exception as e:
                 logger.exception("Could not import migration from %r: %r", self.path, e)
                 raise exceptions.BadMigration(self.path, e)
@@ -310,16 +309,14 @@ class TransactionWrapper(StepBase):
         return "<TransactionWrapper {!r}>".format(self.step)
 
     def apply(self, backend, force=False, direction="apply"):
-        with backend.transaction() as transaction:
-            try:
+        try:
+            with backend.transaction():
                 getattr(self.step, direction)(backend, force)
-            except backend.DatabaseError:
-                if force or self.ignore_errors in (direction, "all"):
-                    logger.exception("Ignored error in %r", self.step)
-                    transaction.rollback()
-                    return
-                else:
-                    raise
+        except backend.DatabaseError:
+            if force or self.ignore_errors in (direction, "all"):
+                logger.exception("Ignored error in %r", self.step)
+            else:
+                raise
 
     def rollback(self, backend, force=False):
         self.apply(backend, force, "rollback")
@@ -451,7 +448,7 @@ class StepGroup(MigrationStep):
             item.rollback(backend, force)
 
 
-def _expand_sources(sources) -> Iterable[Tuple[str, List[str]]]:
+def _expand_sources(sources) -> t.Iterable[t.Tuple[str, t.List[str]]]:
     package_match = re.compile(r"^package:([^\s\/:]+):(.*)$").match
     for source in sources:
         mo = package_match(source)
@@ -482,7 +479,7 @@ def read_migrations(*sources):
     """
     Return a ``MigrationList`` containing all migrations from ``sources``.
     """
-    migrations = OrderedDict()  # type: Dict[str, MigrationList]
+    migrations: t.Dict[str, MigrationList] = OrderedDict()
 
     for source, paths in _expand_sources(sources):
         for path in paths:
@@ -495,7 +492,9 @@ def read_migrations(*sources):
                 migration_class = PostApplyHookMigration
 
             migration = migration_class(
-                os.path.splitext(os.path.basename(path))[0], path, source_dir=source,
+                os.path.splitext(os.path.basename(path))[0],
+                path,
+                source_dir=source,
             )
             ml = migrations.setdefault(source, MigrationList())
             if migration_class is PostApplyHookMigration:
@@ -524,7 +523,7 @@ class MigrationList(abc.MutableSequence):
         return "{}({})".format(self.__class__.__name__, repr(self.items))
 
     def check_conflicts(self):
-        c = Counter()  # type: Dict[str, int]
+        c: t.Dict[str, int] = Counter()
         for item in self:
             c[item.id] += 1
             if c[item.id] > 1:
@@ -599,9 +598,9 @@ class StepCollector(object):
 
         def do_add(use_transactions):
             wrapper = TransactionWrapper if use_transactions else Transactionless
-            t = MigrationStep(next(self.step_id), apply, rollback)  # type: StepBase
-            t = wrapper(t, ignore_errors)
-            return t
+            step: StepBase = MigrationStep(next(self.step_id), apply, rollback)
+            step = wrapper(step, ignore_errors)
+            return step
 
         self.steps[do_add] = 1
         return do_add
@@ -719,72 +718,18 @@ def heads(migration_list):
     return heads
 
 
-def topological_sort(
-    migration_list: Iterable[Migration],
-) -> Iterable[Migration]:
+def topological_sort(migrations: t.Iterable[Migration]) -> t.Iterable[Migration]:
+    from yoyo.topologicalsort import topological_sort as topological_sort_impl
+    from yoyo.topologicalsort import CycleError
 
-    # Make a copy of migration_list. It's probably an iterator.
-    migration_list = list(migration_list)
-
-    # Track graph edges in two parallel data structures.
-    # Use OrderedDict so that we can traverse edges in order
-    # and keep the sort stable
-    forward_edges = defaultdict(
-        OrderedDict
-    )  # type: Dict[Migration, Dict[Migration, int]]
-    backward_edges = defaultdict(
-        OrderedDict
-    )  # type: Dict[Migration, Dict[Migration, int]]
-
-    def sort_by_stability_order(
-        items, ordering={m: index for index, m in enumerate(migration_list)}
-    ):
-        return sorted(
-            (item for item in items if item in ordering), key=ordering.get
+    migration_list = list(migrations)
+    all_migrations = set(migration_list)
+    dependency_graph = {m: (m.depends & all_migrations) for m in migration_list}
+    try:
+        return topological_sort_impl(migration_list, dependency_graph)
+    except CycleError as e:
+        raise exceptions.BadMigration(
+            "Circular dependencies among these migrations {}".format(
+                ", ".join(m.id for m in e.args[1])
+            )
         )
-
-    for m in migration_list:
-        for n in sort_by_stability_order(m.depends):
-            forward_edges[n][m] = 1
-            backward_edges[m][n] = 1
-
-    def check_cycles(item):
-        stack: List[Tuple[Migration, List[Migration]]] = [(item, [])]
-        while stack:
-            n, path = stack.pop()
-            if n in path:
-                raise exceptions.BadMigration(
-                    "Circular dependencies among these migrations {}".format(
-                        ", ".join(m.id for m in path + [n])
-                    )
-                )
-            stack.extend((f, path + [n]) for f in forward_edges[n])
-
-    seen = set()
-    for item in migration_list:
-
-        if item in seen:
-            continue
-
-        check_cycles(item)
-
-        # if item is in a dedepency graph, go back to the root node
-        while backward_edges[item]:
-            item = next(iter(backward_edges[item]))
-
-        # is item at the start of a dependency graph?
-        if forward_edges[item]:
-            stack = [item]
-            while stack:
-                m = stack.pop()
-                yield m
-                seen.add(m)
-                for child in list(reversed(list(forward_edges[m]))):
-                    if all(
-                        dependency in seen
-                        for dependency in backward_edges[child]
-                    ):
-                        stack.append(child)
-        else:
-            yield item
-            seen.add(item)
