@@ -2,14 +2,15 @@ from tempfile import NamedTemporaryFile
 from threading import Thread
 import time
 
-from mock import Mock
-from mock import call
-from mock import patch
+from unittest.mock import Mock
+from unittest.mock import call
+from unittest.mock import patch
 import pytest
 
 from yoyo import backends
 from yoyo import read_migrations
 from yoyo import exceptions
+from yoyo.backends.contrib.redshift import RedshiftBackend
 from yoyo.connections import get_backend
 from yoyo.tests import get_test_backends
 from yoyo.tests import get_test_dburis
@@ -43,11 +44,10 @@ class TestTransactionHandling(object):
         with backend.transaction():
             backend.execute("INSERT INTO yoyo_t values ('A')")
 
-            with backend.transaction() as trans:
+            with backend.transaction(rollback_on_exit=True):
                 backend.execute("INSERT INTO yoyo_t values ('B')")
-                trans.rollback()
 
-            with backend.transaction() as trans:
+            with backend.transaction():
                 backend.execute("INSERT INTO yoyo_t values ('C')")
 
         with backend.transaction():
@@ -74,8 +74,8 @@ class TestTransactionHandling(object):
 
     def test_backend_detects_transactional_ddl(self, backend):
         expected = {
+            RedshiftBackend: True,
             backends.PostgresqlBackend: True,
-            backends.RedshiftBackend: True,
             backends.SQLiteBackend: True,
             backends.MySQLBackend: False,
         }
@@ -94,21 +94,16 @@ class TestTransactionHandling(object):
         if backend.has_transactional_ddl:
             return
 
-        with backend.transaction() as trans:
+        with backend.transaction(rollback_on_exit=True):
             backend.execute("CREATE TABLE yoyo_a (id INT)")  # implicit commit
             backend.execute("INSERT INTO yoyo_a VALUES (1)")
             backend.execute("CREATE TABLE yoyo_b (id INT)")  # implicit commit
             backend.execute("INSERT INTO yoyo_b VALUES (1)")
-            trans.rollback()
 
-        count_a = backend.execute("SELECT COUNT(1) FROM yoyo_a").fetchall()[0][
-            0
-        ]
+        count_a = backend.execute("SELECT COUNT(1) FROM yoyo_a").fetchall()[0][0]
         assert count_a == 1
 
-        count_b = backend.execute("SELECT COUNT(1) FROM yoyo_b").fetchall()[0][
-            0
-        ]
+        count_b = backend.execute("SELECT COUNT(1) FROM yoyo_b").fetchall()[0][0]
         assert count_b == 0
 
     def test_statements_requiring_no_transaction(self):
@@ -183,19 +178,14 @@ class TestConcurrency(object):
         return lock_sleep
 
     def skip_if_not_concurrency_safe(self, backend):
-        if (
-            "sqlite" in backend.uri.scheme
-            and backend.uri.database == ":memory:"
-        ):
+        if "sqlite" in backend.uri.scheme and backend.uri.database == ":memory:":
             pytest.skip(
                 "Concurrency tests not supported for SQLite "
                 "in-memory databases, which cannot be shared "
                 "between threads"
             )
         if backend.driver.threadsafety < 1:
-            pytest.skip(
-                "Concurrency tests not supported for non-threadsafe backends"
-            )
+            pytest.skip("Concurrency tests not supported for non-threadsafe backends")
 
     def test_lock(self, dburi):
         """
@@ -310,9 +300,7 @@ class TestInitConnection(object):
 
         finally:
             with backend.transaction():
-                backend.execute(
-                    "ALTER DATABASE {} RESET SEARCH_PATH".format(dbname)
-                )
+                backend.execute("ALTER DATABASE {} RESET SEARCH_PATH".format(dbname))
                 backend.execute("DROP SCHEMA custom_schema CASCADE")
 
     def test_postgresql_migrations_can_change_schema_search_path(self):
@@ -324,9 +312,7 @@ class TestInitConnection(object):
             pytest.skip("PostgreSQL backend not available")
         backend = get_backend(dburi)
         with migrations_dir(
-            **{
-                "1.sql": "SELECT pg_catalog.set_config('search_path', '', false)"
-            }
+            **{"1.sql": "SELECT pg_catalog.set_config('search_path', '', false)"}
         ) as tmpdir:
             migrations = read_migrations(tmpdir)
             backend.apply_migrations(migrations)
