@@ -17,6 +17,7 @@ from datetime import datetime
 from tempfile import mkdtemp
 from functools import partial
 from itertools import count
+import pathlib
 import io
 import os
 import os.path
@@ -24,8 +25,8 @@ import re
 import sys
 import textwrap
 
-from mock import Mock, patch, call
-import frozendate
+from unittest.mock import Mock, patch, call
+import freezegun
 import pytest
 import tms
 
@@ -39,13 +40,13 @@ from yoyo.scripts import newmigration
 
 
 def is_tmpfile(p, directory=None):
-    return (
-        p.startswith(directory) if directory else True
-    ) and os.path.basename(p).startswith(newmigration.tempfile_prefix)
+    return (p.startswith(directory) if directory else True) and os.path.basename(
+        p
+    ).startswith(newmigration.tempfile_prefix)
 
 
 class TestInteractiveScript(object):
-    def setup(self):
+    def setup_method(self):
         self.stdout_tty_patch = patch("sys.stdout.isatty", return_value=True)
         self.stdout_tty_patch.start()
         self.confirm_patch = patch("yoyo.utils.confirm", return_value=False)
@@ -57,7 +58,7 @@ class TestInteractiveScript(object):
         self.saved_cwd = os.getcwd()
         os.chdir(self.tmpdir)
 
-    def teardown(self):
+    def teardown_method(self):
         self.prompt_patch.stop()
         self.confirm_patch.stop()
         self.stdout_tty_patch.stop()
@@ -127,9 +128,9 @@ class TestYoyoScript(TestInteractiveScript):
 
     def test_it_prompts_password(self, tmpdir):
         dburi = "sqlite://user@/:memory"
-        with patch(
-            "yoyo.scripts.main.getpass", return_value="fish"
-        ) as getpass, patch("yoyo.connections.get_backend") as get_backend:
+        with patch("yoyo.scripts.main.getpass", return_value="fish") as getpass, patch(
+            "yoyo.connections.get_backend"
+        ) as get_backend:
             main(["apply", str(tmpdir), "--database", dburi, "--prompt-password"])
             assert getpass.call_count == 1
             assert get_backend.call_args == call(
@@ -189,9 +190,7 @@ class TestYoyoScript(TestInteractiveScript):
 
         self.confirm.return_value = True
         main(["apply", str(tmpdir)])
-        prompts = [
-            args[0].lower() for args, kwargs in self.confirm.call_args_list
-        ]
+        prompts = [args[0].lower() for args, kwargs in self.confirm.call_args_list]
         assert len(prompts) == 2
         assert prompts[0].startswith("move legacy configuration")
         assert prompts[1].startswith("delete legacy configuration")
@@ -243,9 +242,7 @@ class TestYoyoScript(TestInteractiveScript):
             run_migrations = partial(
                 main, ["apply", "-b", tmpdir, "--database", str(backend.uri)]
             )
-            threads = [
-                threading.Thread(target=run_migrations) for ix in range(20)
-            ]
+            threads = [threading.Thread(target=run_migrations) for ix in range(20)]
             for t in threads:
                 t.start()
             for t in threads:
@@ -266,16 +263,14 @@ class TestYoyoScript(TestInteractiveScript):
         )
         backend.commit()
         main(["break-lock", "--database", dburi])
-        lock_count = backend.execute(
-            "SELECT COUNT(1) FROM yoyo_lock"
-        ).fetchone()[0]
+        lock_count = backend.execute("SELECT COUNT(1) FROM yoyo_lock").fetchone()[0]
         assert lock_count == 0
 
     def test_it_prompts_password_on_break_lock(self):
         dburi = "sqlite://user@/:memory"
-        with patch(
-            "yoyo.scripts.main.getpass", return_value="fish"
-        ) as getpass, patch("yoyo.connections.get_backend") as get_backend:
+        with patch("yoyo.scripts.main.getpass", return_value="fish") as getpass, patch(
+            "yoyo.connections.get_backend"
+        ) as get_backend:
             main(["break-lock", "--database", dburi, "--prompt-password"])
             assert getpass.call_count == 1
             assert get_backend.call_args == call(
@@ -331,9 +326,7 @@ class TestMarkCommand(TestInteractiveScript):
             backend = get_backend(self.dburi)
             backend.apply_migrations(migrations[:1])
 
-            with patch(
-                "yoyo.scripts.migrate.prompt_migrations"
-            ) as prompt_migrations:
+            with patch("yoyo.scripts.migrate.prompt_migrations") as prompt_migrations:
                 main(["mark", tmpdir, "--database", self.dburi])
                 _, prompted, _ = prompt_migrations.call_args[0]
                 prompted = [m.id for m in prompted]
@@ -373,9 +366,7 @@ class TestUnmarkCommand(TestInteractiveScript):
             backend.apply_migrations(migrations[:2])
             assert len(backend.get_applied_migration_hashes()) == 2
 
-            with patch(
-                "yoyo.scripts.migrate.prompt_migrations"
-            ) as prompt_migrations:
+            with patch("yoyo.scripts.migrate.prompt_migrations") as prompt_migrations:
                 main(["unmark", tmpdir, "--database", self.dburi])
                 _, prompted, _ = prompt_migrations.call_args[0]
                 prompted = [m.id for m in prompted]
@@ -399,19 +390,19 @@ class TestUnmarkCommand(TestInteractiveScript):
 
 
 class TestNewMigration(TestInteractiveScript):
-    def setup(self):
+    def setup_method(self):
         def mockstat(f, c=count()):
             return Mock(st_mtime=next(c))
 
-        super(TestNewMigration, self).setup()
+        super(TestNewMigration, self).setup_method()
         self.subprocess_patch = patch("yoyo.scripts.newmigration.subprocess")
         self.subprocess = self.subprocess_patch.start()
         self.subprocess.call.return_value = 0
         self.stat_patch = patch("yoyo.scripts.newmigration.stat", mockstat)
         self.stat_patch.start()
 
-    def teardown(self):
-        super(TestNewMigration, self).teardown()
+    def teardown_method(self):
+        super(TestNewMigration, self).teardown_method()
         self.subprocess_patch.stop()
         self.stat_patch.stop()
 
@@ -429,13 +420,9 @@ class TestNewMigration(TestInteractiveScript):
                 assert "__depends__ = {'m2', 'm3'}" in f.read()
 
     def test_it_names_file_by_date_and_sequence(self, tmpdir):
-        with frozendate.freeze(2001, 1, 1):
-            main(
-                ["new", "-b", "-m", "foo", str(tmpdir), "--database", dburi_sqlite3]
-            )
-            main(
-                ["new", "-b", "-m", "bar", str(tmpdir), "--database", dburi_sqlite3]
-            )
+        with freezegun.freeze_time("2001-1-1"):
+            main(["new", "-b", "-m", "foo", str(tmpdir), "--database", dburi_sqlite3])
+            main(["new", "-b", "-m", "bar", str(tmpdir), "--database", dburi_sqlite3])
         names = [n for n in sorted(os.listdir(str(tmpdir))) if n.endswith(".py")]
         assert names[0].startswith("20010101_01_")
         assert names[0].endswith("-foo.py")
@@ -458,9 +445,7 @@ class TestNewMigration(TestInteractiveScript):
         # default to $VISUAL
         with patch("os.environ", {"EDITOR": "ed", "VISUAL": "visualed"}):
             main(["new", str(tmpdir), "--database", dburi_sqlite3])
-            assert self.subprocess.call.call_args == call(
-                ["visualed", tms.Unicode()]
-            )
+            assert self.subprocess.call.call_args == call(["visualed", tms.Unicode()])
 
         # fallback to $EDITOR
         with patch("os.environ", {"EDITOR": "ed"}):
@@ -494,9 +479,7 @@ class TestNewMigration(TestInteractiveScript):
 
         self.subprocess.call = write_migration
         main(["new", str(tmpdir), "--database", dburi_sqlite3])
-        prompts = [
-            args[0].lower() for args, kwargs in self.prompt.call_args_list
-        ]
+        prompts = [args[0].lower() for args, kwargs in self.prompt.call_args_list]
         assert "retry editing?" in prompts[0]
 
     def test_it_defaults_docstring_to_message(self, tmpdir):
@@ -512,18 +495,14 @@ class TestNewMigration(TestInteractiveScript):
             ]
         )
         names = [n for n in sorted(os.listdir(tmpdir)) if n.endswith(".py")]
-        with open(
-            os.path.join(str(tmpdir), names[0]), "r", encoding="utf-8"
-        ) as f:
+        with open(os.path.join(str(tmpdir), names[0]), "r", encoding="utf-8") as f:
             assert "your ad here" in f.read()
 
     def test_it_calls_post_create_command(self, tmpdir):
         self.writeconfig(post_create_command="/bin/ls -l {} {}")
-        with frozendate.freeze(2001, 1, 1):
+        with freezegun.freeze_time("2001-1-1"):
             main(["new", "-b", str(tmpdir), "--database", dburi_sqlite3])
-        is_filename = tms.Str(
-            lambda s: os.path.basename(s).startswith("20010101_01_")
-        )
+        is_filename = tms.Str(lambda s: os.path.basename(s).startswith("20010101_01_"))
         assert self.subprocess.call.call_args == call(
             ["/bin/ls", "-l", is_filename, is_filename]
         )
@@ -618,3 +597,25 @@ class TestDevelopCommand(TestInteractiveScript):
                 ("m1", "apply"),
                 ("m2", "apply"),
             ]
+
+
+class TestInitCommand(TestInteractiveScript):
+    def test_it_creates_project(self):
+        main(["-b", "init", "migrations", "--database", "sqlite://foo"])
+        config = (pathlib.Path(self.tmpdir) / "yoyo.ini").read_text()
+        assert "database = sqlite://foo" in config
+        assert "sources = migrations" in config
+        assert (pathlib.Path(self.tmpdir) / "migrations").is_dir()
+
+    def test_it_doesnt_overwrite(self):
+        configfile = pathlib.Path(self.tmpdir) / "yoyo.ini"
+        configfile.write_text("[existing file]")
+        main(["-b", "init", "migrations", "--database", "sqlite://foo"])
+        assert "database = sqlite://foo" not in configfile.read_text()
+        assert "existing file" in configfile.read_text()
+        assert not (pathlib.Path(self.tmpdir) / "migrations").exists()
+
+    def test_it_doesnt_prompt(self):
+        with patch("yoyo.scripts.main.prompt_save_config") as prompt_save_config:
+            main(["-b", "init", "migrations", "--database", "sqlite://foo"])
+            assert prompt_save_config.called is False
